@@ -1,21 +1,19 @@
 package XML::MyXML;
 {
-  $XML::MyXML::VERSION = '0.1003';
+  $XML::MyXML::VERSION = '0.9000';
 }
 # ABSTRACT: A simple-to-use XML module, for parsing and creating XML documents
 
 use strict;
 use warnings;
-use utf8;
 use Carp;
+use Scalar::Util qw( weaken );
 require Exporter;
 our @ISA = qw(Exporter);
 our @EXPORT_OK = qw(tidy_xml object_to_xml xml_to_object simple_to_xml xml_to_simple check_xml xml_escape);
 our %EXPORT_TAGS = (all => [@EXPORT_OK]);
 use Encode;
 
-warn "WARNING: Your program is using XML::MyXML, which is deprecated. Development on this module has stopped, and the module will be removed from future CPAN releases on January 1st, 2016. Please start using XML::MyXML::II as soon as possible instead, which you have installed on your system. Please also make sure you read https://metacpan.org/pod/XML::MyXML::II#DIFFERENCES-FROM-XML::MyXML to see the few differences between XML::MyXML and XML::MyXML::II";
-
 
 sub _encode {
 	my $string = shift;
@@ -53,26 +51,38 @@ sub _decode {
 					'\'' => '&apos;',
 					'"' => '&quot;',
 	);
-	Encode::_utf8_on($string);
-	if (utf8::valid($string)) {
-		$string =~ s/\&\#x([0-9a-f]+)\;/chr(hex($1))/egi;
-		$string =~ s/\&\#([0-9]+)\;/chr($1)/eg;
+	my @capture = map "\Q$_\E", keys %replace;
+	push @capture, '&#x[0-9A-Fa-f]+;', '&#[0-9]+;';
+	my $capture = "(".join("|", @capture).")";
+	my @captured = $string =~ /$capture/g;
+	@captured	or return $string;
+	my %conv;
+	foreach my $e (@captured) {
+		if (exists $conv{$e}) { next; }
+		if (exists $replace{$e}) {
+			$conv{$e} = $replace{$e};
+		} elsif ($e =~ /^&#x([0-9a-fA-F]+);$/) {
+			$conv{$e} = chr(hex($1));
+		} elsif ($e =~ /^&#([0-9]+);$/) {
+			$conv{$e} = chr($1);
+		}
 	}
-	Encode::_utf8_off($string);
-	my $keys = "(".join("|", keys %replace).")";
-	$string =~ s/$keys/$replace{$1}/g;
+	my $keys = "(".join("|", map "\Q$_\E", keys %conv).")";
+	$string =~ s/$keys/$conv{$1}/g;
 	return $string;
 }
 
 sub _strip {
 	my $string = shift;
 
+	# NOTE: Replace this with the 'r' flag of the substitution operator
 	return defined $string ? ($string =~ /^\s*(.*?)\s*$/s)[0] : $string;
 }
 
 sub _strip_ns {
 	my $string = shift;
 
+	# NOTE: Replace this with the 'r' flag of the substitution operator
 	return defined $string ? ($string =~ /^(?:.+\:)?(.*)$/s)[0] : $string;
 }
 
@@ -80,14 +90,12 @@ sub _strip_ns {
 
 sub tidy_xml {
 	my $xml = shift;
-	if ($xml eq 'XML::MyXML') { confess "Error: 'tidy_xml' is a function, not a method"; }
 	my $flags = shift || {};
 
 	my $object = xml_to_object($xml, $flags);
 	defined $object or return $object;
 	_tidy_object($object, undef, $flags);
 	my $return = $object->to_xml({ %$flags, tidy => 0 }) . "\n";
-	$object->delete();
 	return $return;
 }
 
@@ -97,25 +105,19 @@ sub xml_to_object {
 	my $xml = shift;
 	my $flags = (@_ and defined $_[0]) ? $_[0] : {};
 
-	my $soft = $flags->{'soft'}; # soft = 'don't die if can't parse, just return undef'
-
 	if ($flags->{'file'}) {
-		open my $fh, '<', $xml or do { confess "Error: The file '$xml' could not be opened for reading: $!" unless $soft; return undef; };
+		open my $fh, '<', $xml	or confess "Error: The file '$xml' could not be opened for reading: $!";
 		$xml = join '', <$fh>;
 		close $fh;
 	}
 
 	my (undef, undef, $encoding) = $xml =~ /<\?xml(\s[^>]+)?\sencoding=(['"])(.*?)\2/g;
-	Encode::_utf8_on($xml);
-	if (! utf8::valid($xml)) {
-		if ($encoding and $encoding !~ /^utf-?8$/i) {
-			Encode::_utf8_off($xml);
-			$xml = decode($encoding, $xml);
-			Encode::_utf8_on($xml);
-		}
-	}
-	if (! utf8::valid($xml)) { confess "Error: Input string is invalid UTF-8" unless $soft; return undef; }
-	Encode::_utf8_off($xml);
+	$encoding = 'UTF-8'		if ! defined $encoding;
+	if ($encoding =~ /^utf-?8$/i) { $encoding = 'UTF-8'; }
+	eval {
+		$xml = decode($encoding, $xml, Encode::FB_CROAK);
+	};
+	! $@	or confess 'Error: Input string is invalid UTF-8';
 
 	my $entities = {};
 
@@ -127,10 +129,10 @@ sub xml_to_object {
 		my $init_ws = 1;
 		foreach my $el (@els) {
 			if ($el =~ /^<!--/) {
-				if ($el !~ /-->$/) { confess "Error: unclosed XML comment block - '$el'" unless $soft; return undef; }
+				if ($el !~ /-->$/) { confess encode_utf8("Error: unclosed XML comment block - '$el'"); }
 				undef $el;
 			} elsif ($el =~ /^<\?/) { # like <?xml?> or <?target?>
-				if ($el !~ /\?>$/) { confess "Error: Erroneous special markup - '$el'" unless $soft; return undef; }
+				if ($el !~ /\?>$/) { confess encode_utf8("Error: Erroneous special markup - '$el'"); }
 				undef $el;
 			} elsif (my ($entname, undef, $entvalue) = $el =~ /^<!ENTITY\s+(\S+)\s+(['"])(.*?)\2\s*>$/g) {
 				$entities->{"&$entname;"} = _decode($entvalue);
@@ -146,18 +148,18 @@ sub xml_to_object {
 			}
 		}
 		@els = grep { defined $_ } @els;
-		if (! @els) { confess "Error: No elements in XML document" unless $soft; return undef; }
+		if (! @els) { confess "Error: No elements in XML document"; }
 	}
 	my @stack;
 	my $object = bless ({ content => [] }, 'XML::MyXML::Object');
 	my $pointer = $object;
 	foreach my $el (@els) {
 		if ($el =~ /^<\/?>$/) {
-			confess "Error: Strange element: '$el'" unless $soft; $object->delete(); return undef;
+			confess encode_utf8("Error: Strange element: '$el'");
 		} elsif ($el =~ /^<\/[^\s>]+>$/) {
 			my ($element) = $el =~ /^<\/(\S+)>$/g;
-			if (! length($element)) { confess "Error: Strange element: '$el'" unless $soft; $object->delete(); return undef; }
-			if ($stack[$#stack]->{'element'} ne $element) { confess "Error: Incompatible stack element: stack='".$stack[$#stack]->{'element'}."' element='$el'" unless $soft; $object->delete(); return undef; }
+			if (! length($element)) { confess encode_utf8("Error: Strange element: '$el'"); }
+			if ($stack[-1]{'element'} ne $element) { confess encode_utf8("Error: Incompatible stack element: stack='".$stack[-1]{'element'}."' element='$el'"); }
 			my $stackentry = pop @stack;
 			if ($#{$stackentry->{'content'}} == -1) {
 				delete $stackentry->{'content'};
@@ -165,7 +167,7 @@ sub xml_to_object {
 			$pointer = $stackentry->{'parent'};
 		} elsif ($el =~ /^<[^>]+\/>$/) {
 			my ($element) = $el =~ /^<([^\s>\/]+)/g;
-			if (! length($element)) { confess "Error: Strange element: '$el'" unless $soft; $object->delete(); return undef; }
+			if (! length($element)) { confess encode_utf8("Error: Strange element: '$el'"); }
 			my $elementmeta = quotemeta($element);
 			$el =~ s/^<$elementmeta//;
 			$el =~ s/\/>$//;
@@ -175,15 +177,16 @@ sub xml_to_object {
 			my %attr;
 			foreach my $attr (@attrs) {
 				my ($name, undef, $value) = $attr =~ /^(\S+?)=(['"])(.*?)\2$/g;
-				if (! length($name) or ! defined($value)) { confess "Error: Strange attribute: '$attr'" unless $soft; $object->delete(); return undef; }
+				if (! length($name) or ! defined($value)) { confess encode_utf8("Error: Strange attribute: '$attr'"); }
 				$attr{$name} = _decode($value, $entities);
 			}
 			my $entry = { element => $element, attrs => \%attr, parent => $pointer };
+			weaken( $entry->{'parent'} );
 			bless $entry, 'XML::MyXML::Object';
 			push @{$pointer->{'content'}}, $entry;
 		} elsif ($el =~ /^<[^\s>\/][^>]*>$/) {
 			my ($element) = $el =~ /^<([^\s>]+)/g;
-			if (! length($element)) { confess "Error: Strange element: '$el'" unless $soft; $object->delete(); return undef; }
+			if (! length($element)) { confess encode_utf8("Error: Strange element: '$el'"); }
 			my $elementmeta = quotemeta($element);
 			$el =~ s/^<$elementmeta//;
 			$el =~ s/>$//;
@@ -193,23 +196,25 @@ sub xml_to_object {
 			my %attr;
 			foreach my $attr (@attrs) {
 				my ($name, undef, $value) = $attr =~ /^(\S+?)=(['"])(.*?)\2$/g;
-				if (! length($name) or ! defined($value)) { confess "Error: Strange attribute: '$attr'" unless $soft; $object->delete(); return undef; }
+				if (! length($name) or ! defined($value)) { confess encode_utf8("Error: Strange attribute: '$attr'"); }
 				$attr{$name} = _decode($value, $entities);
 			}
 			my $entry = { element => $element, attrs => \%attr, content => [], parent => $pointer };
+			weaken( $entry->{'parent'} );
 			bless $entry, 'XML::MyXML::Object';
 			push @stack, $entry;
 			push @{$pointer->{'content'}}, $entry;
 			$pointer = $entry;
 		} elsif ($el =~ /^[^<>]*$/) {
 			my $entry = { value => _decode($el, $entities), parent => $pointer };
+			weaken( $entry->{'parent'} );
 			bless $entry, 'XML::MyXML::Object';
 			push @{$pointer->{'content'}}, $entry;
 		} else {
-			confess "Error: Strange element: '$el'" unless $soft; $object->delete(); return undef;
+			confess encode_utf8("Error: Strange element: '$el'");
 		}
 	}
-	if (@stack) { confess "Error: The <$stack[-1]->{'element'}> element has not been closed in XML" unless $soft; $object->delete(); return undef; }
+	if (@stack) { confess encode_utf8("Error: The <$stack[-1]{'element'}> element has not been closed in XML"); }
 	$object = $object->{'content'}[0];
 	$object->{'parent'} = undef;
 	return $object;
@@ -273,10 +278,14 @@ sub _tidy_object {
 	@children = @{$object->{'content'}};
 	$object->{'content'} = [];
 	for my $i (0..$#children) {
-		push @{$object->{'content'}}, bless ({ value => "\n".($flags->{'indentstring'}x($tabs+1)), parent => $object }, 'XML::MyXML::Object');
+		my $whitespace = bless ({ value => "\n".($flags->{'indentstring'}x($tabs+1)), parent => $object }, 'XML::MyXML::Object');
+		weaken( $whitespace->{'parent'} );
+		push @{$object->{'content'}}, $whitespace;
 		push @{$object->{'content'}}, $children[$i];
 	}
-	push @{$object->{'content'}}, bless ({ value => "\n".($flags->{'indentstring'}x($tabs)), parent => $object }, 'XML::MyXML::Object');
+	my $whitespace = bless ({ value => "\n".($flags->{'indentstring'}x($tabs)), parent => $object }, 'XML::MyXML::Object');
+	weaken( $whitespace->{'parent'} );
+	push @{$object->{'content'}}, $whitespace;
 
 	for my $i (0..$#{$object->{'content'}}) {
 		_tidy_object($object->{'content'}[$i], $tabs+1, $flags);
@@ -291,13 +300,11 @@ sub simple_to_xml {
 
 	my $xml = '';
 	my ($key, $value, @residue) = (ref $arref eq 'HASH') ? %$arref : @$arref;
-	Encode::_utf8_off($key);
 	if (@residue) { confess "Error: the provided simple ref contains more than 1 top element"; }
 	my ($tag) = $key =~ /^(\S+)/g;
-	confess "Error: Strange key: $key" if ! defined $tag;
+	confess encode_utf8("Error: Strange key: $key") if ! defined $tag;
 
 	if (! ref $value) {
-		Encode::_utf8_off($value);
 		if (defined $value and length $value) {
 			$xml .= "<$key>"._encode($value)."</$tag>";
 		} else {
@@ -310,16 +317,15 @@ sub simple_to_xml {
 	my $decl = $flags->{'complete'} ? '<?xml version="1.0" encoding="UTF-8" standalone="yes" ?>'."\n" : '';
 	$decl .= "<?xml-stylesheet type=\"text/xsl\" href=\"$flags->{'xslt'}\"?>\n" if $flags->{'xslt'};
 	$xml = $decl . $xml;
-	if ($flags->{'utf8'}) { Encode::_utf8_on($xml); }
 
 	if (defined $flags->{'save'}) {
 		open my $fh, '>', $flags->{'save'} or confess "Error: Couldn't open file '$flags->{'save'}' for writing: $!";
-		if ($flags->{'utf8'}) { binmode $fh, ':encoding(UTF-8)'; }
+		binmode $fh, ':encoding(UTF-8)';
 		print $fh $xml;
 		close $fh;
 	}
 
-	return $xml;
+	return encode_utf8( $xml );
 }
 
 
@@ -334,18 +340,15 @@ sub _arrayref_to_xml {
 	foreach (my $i = 0; $i <= $#$arref; ) {
 	#while (@$arref) {
 		my $key = $arref->[$i++];
-		Encode::_utf8_off($key);
 		#my $key = shift @$arref;
 		my ($tag) = $key =~ /^(\S+)/g;
-		confess "Error: Strange key: $key" if ! defined $tag;
+		confess encode_utf8("Error: Strange key: $key") if ! defined $tag;
 		my $value = $arref->[$i++];
 		#my $value = shift @$arref;
 
 		if ($key eq '!as_is') {
-			Encode::_utf8_off($value);
 			$xml .= $value if check_xml($value);
 		} elsif (! ref $value) {
-			Encode::_utf8_off($value);
 			if (defined $value and length $value) {
 				$xml .= "<$key>"._encode($value)."</$tag>";
 			} else {
@@ -366,15 +369,12 @@ sub _hashref_to_xml {
 	my $xml = '';
 
 	while (my ($key, $value) = each %$hashref) {
-		Encode::_utf8_off($key);
 		my ($tag) = $key =~ /^(\S+)/g;
-		confess "Error: Strange key: $key" if ! defined $tag;
+		confess encode_utf8("Error: Strange key: $key") if ! defined $tag;
 
 		if ($key eq '!as_is') {
-			Encode::_utf8_off($value);
 			$xml .= $value if check_xml($value);
 		} elsif (! ref $value) {
-			Encode::_utf8_off($value);
 			if (defined $value and length $value) {
 				$xml .= "<$key>"._encode($value)."</$tag>";
 			} else {
@@ -396,8 +396,6 @@ sub xml_to_simple {
 
 	my $return = defined $object ? $object->simplify($flags) : $object;
 
-	$object->delete();
-
 	return $return;
 }
 
@@ -425,13 +423,11 @@ sub _objectarray_to_simple_hashref {
 	foreach my $stuff (@$object) {
 		if (defined $stuff->{'element'}) {
 			my $key = $stuff->{'element'};
-			if ($flags->{'strip_ns'}) { $key = XML::MyXML::_strip_ns($key); }
-			Encode::_utf8_on($key) if $flags->{'utf8'};
+			if ($flags->{'strip_ns'}) { $key = _strip_ns($key); }
 			$hashref->{ $key } = _objectarray_to_simple($stuff->{'content'}, $flags);
 		} elsif (defined $stuff->{'value'}) {
 			my $value = $stuff->{'value'};
-			if ($flags->{'strip'}) { $value = XML::MyXML::_strip($value); }
-			Encode::_utf8_on($value) if $flags->{'utf8'};
+			if ($flags->{'strip'}) { $value = _strip($value); }
 			return $value if $value =~ /\S/;
 		}
 	}
@@ -447,8 +443,6 @@ sub _objectarray_to_simple_arrayref {
 	my $object = shift;
 	my $flags = (@_ and defined $_[0]) ? $_[0] : {};
 
-	if (ref $flags ne 'HASH') { confess "Error: This method of setting flags is deprecated in XML::MyXML v0.083 - check module's documentation for the new way"; }
-
 	if (! defined $object) { return undef; }
 
 	my $arrayref = [];
@@ -456,14 +450,12 @@ sub _objectarray_to_simple_arrayref {
 	foreach my $stuff (@$object) {
 		if (defined $stuff->{'element'}) {
 			my $key = $stuff->{'element'};
-			if ($flags->{'strip_ns'}) { $key = XML::MyXML::_strip_ns($key); }
-			Encode::_utf8_on($key) if $flags->{'utf8'};
+			if ($flags->{'strip_ns'}) { $key = _strip_ns($key); }
 			push @$arrayref, ( $key, _objectarray_to_simple($stuff->{'content'}, $flags) );
 			#$hashref->{ $key } = _objectarray_to_simple($stuff->{'content'}, $flags);
 		} elsif (defined $stuff->{'value'}) {
 			my $value = $stuff->{'value'};
-			if ($flags->{'strip'}) { $value = XML::MyXML::_strip($value); }
-			Encode::_utf8_on($value) if $flags->{'utf8'};
+			if ($flags->{'strip'}) { $value = _strip($value); }
 			return $value if $value =~ /\S/;
 		}
 	}
@@ -481,25 +473,19 @@ sub check_xml {
 	my $xml = shift;
 	my $flags = (@_ and defined $_[0]) ? $_[0] : {};
 
-	if (ref $flags ne 'HASH') { confess "Error: This method of setting flags is deprecated in XML::MyXML v0.083 - check module's documentation for the new way"; }
-
-	my $obj = xml_to_object($xml, { %$flags, soft => 1 });
-	if ($obj) {
-		$obj->delete();
-		return 1;
-	} else {
-		return 0;
-	}
+	my $obj = eval { xml_to_object($xml, $flags) };
+	return ! $@;
 }
 
 
 
 package XML::MyXML::Object;
 {
-  $XML::MyXML::Object::VERSION = '0.1003';
+  $XML::MyXML::Object::VERSION = '0.9000';
 }
 
 use Carp;
+use Encode;
 
 
 sub new {
@@ -599,7 +585,6 @@ sub value {
 	if ($self->{'content'} and $self->{'content'}[0]) {
 		my $value = $self->{'content'}[0]{'value'};
 		if ($flags->{'strip'}) { $value = XML::MyXML::_strip($value); }
-		Encode::_utf8_on($value) if $flags->{'utf8'};
 		return $value;
 	} else {
 		return undef;
@@ -615,7 +600,6 @@ sub attr {
 		my $next = shift;
 		if (! ref $next) {
 			$set_to = $next;
-			Encode::_utf8_off($set_to);
 			$must_set = 1;
 			$flags = shift;
 		} else {
@@ -628,7 +612,6 @@ sub attr {
 		if ($must_set) {
 			if (defined ($set_to)) {
 				$self->{'attrs'}{$attrname} = $set_to;
-				Encode::_utf8_on($set_to) if $flags->{'utf8'};
 				return $set_to;
 			} else {
 				delete $self->{'attrs'}{$attrname};
@@ -636,17 +619,10 @@ sub attr {
 			}
 		} else {
 			my $attrvalue = $self->{'attrs'}->{$attrname};
-			Encode::_utf8_on($attrvalue) if $flags->{'utf8'};
 			return $attrvalue;
 		}
 	} else {
-		my %attr = %{$self->{'attrs'}};
-		if ($flags->{'utf8'}) {
-			foreach my $key (keys %attr) {
-				Encode::_utf8_on($attr{$key});
-			}
-		}
-		return %attr;
+		return %{$self->{'attrs'}};
 	}
 }
 
@@ -657,8 +633,7 @@ sub tag {
 
 	my $tag = $self->{'element'};
 	if (defined $tag) {
-		$tag =~ s/^.*\://	unless exists $flags->{'strip_ns'} and ! $flags->{'strip_ns'};
-		Encode::_utf8_on($tag) if $flags->{'utf8'};
+		$tag =~ s/^.*\://	if $flags->{'strip_ns'};
 		return $tag;
 	} else {
 		return undef;
@@ -670,8 +645,6 @@ sub simplify {
 	my $self = shift;
 	my $flags = (@_ and defined $_[0]) ? $_[0] : {};
 
-	if (ref $flags ne 'HASH') { confess "Error: This method of setting flags is deprecated in XML::MyXML v0.083 - check module's documentation for the new way"; }
-
 	my $simple = XML::MyXML::_objectarray_to_simple([$self], $flags);
 	if (! $flags->{'internal'}) {
 		return $simple;
@@ -690,13 +663,11 @@ sub to_xml {
 	my $flags = shift || {};
 
 	my $decl = $flags->{'complete'} ? '<?xml version="1.1" encoding="UTF-8" standalone="yes" ?>'."\n" : '';
-	my $xml = XML::MyXML::_objectarray_to_xml([$self]);
+	my $xml = encode_utf8( XML::MyXML::_objectarray_to_xml([$self]) );
 	if ($flags->{'tidy'}) { $xml = XML::MyXML::tidy_xml($xml, { %$flags, complete => 0, save => undef }); }
 	$xml = $decl . $xml;
-	if ($flags->{'utf8'}) { Encode::_utf8_on($xml); }
 	if (defined $flags->{'save'}) {
 		open my $fh, '>', $flags->{'save'} or confess "Error: Couldn't open file '$flags->{'save'}' for writing: $!";
-		if ($flags->{'utf8'}) { binmode $fh, ':encoding(UTF-8)'; }
 		print $fh $xml;
 		close $fh;
 	}
@@ -716,33 +687,6 @@ sub to_tidy_xml {
 
 
 
-sub delete {
-	my $self = shift;
-
-	# Remove self from parent's "content" field
-	my $parent = $self->{'parent'};
-	if ($parent) {
-		my $content = $parent->{'content'};
-		if ($content) {
-			my @new = ();
-			foreach my $item (@$content) {
-				if ($item != $self) { push @new, $item; }
-			}
-			$parent->{'content'} = \@new;
-		}
-	}
-
-	my $content = $self->{'content'};
-	if ($content) {
-		foreach my $item (@$content) {
-			$item->delete();
-		}
-	}
-
-	delete $self->{$_} foreach keys %$self;
-}
-
-
 1; # End of XML::MyXML
 
 __END__
@@ -755,7 +699,7 @@ XML::MyXML - A simple-to-use XML module, for parsing and creating XML documents
 
 =head1 VERSION
 
-version 0.1003
+version 0.9000
 
 =head1 SYNOPSIS
 
@@ -771,12 +715,6 @@ version 0.1003
     $obj->simplify is hashref { item => { name => 'Table', price => { usd => '10.00', eur => '8.50' } } }
     $obj->simplify({ internal => 1 }) is hashref { name => 'Table', price => { usd => '10.00', eur => '8.50' } }
 
-=head1 PLEASE USE XML::MyXML::II INSTEAD
-
-B<< *** This module will not be maintained anymore *** Please use L<XML::MyXML::II> instead, which is also contained in this distribution *** >>
-
-It's main improvements are: better unicode handling and automatic object destruction
-
 =head1 EXPORT
 
 tidy_xml, xml_to_object, object_to_xml, simple_to_xml, xml_to_simple, check_xml
@@ -787,7 +725,7 @@ This module can parse XML comments, CDATA sections, XML entities (the standard f
 
 It will ignore (won't parse) C<< <!DOCTYPE...> >>, C<< <?...?> >> and other C<< <!...> >> special markup
 
-Parsed documents must be UTF-8 encoded unless an encoding is declared in the initial XML declaration <?xml ... ?> of the document. All XML documents produced by this module will be UTF-8 encoded, as will be all strings output by its functions.
+XML documents passed as parameters to this module's functions must be strings containing bytes/octets, rather than contain characters. They also must be UTF-8 encoded unless an encoding is declared in the initial XML declaration <?xml ... ?> of the document. All XML documents produced by this module will be UTF-8 encoded (bytes/octets). However all other strings which are output by this module's functions and methods (and which are not XML documents) will contain characters rather than bytes/octets.
 
 XML documents to be parsed may not contain the C<< > >> character unencoded in attribute values
 
@@ -801,8 +739,6 @@ C<file> : the function will expect the path to a file containing an XML document
 
 C<complete> : the function's XML output will include an XML declaration (C<< <?xml ... ?>  >>) in the beginning
 
-C<soft> : the function will return undef instead of dying in case of an error during XML parsing
-
 C<internal> : the function will only return the contents of an element in a hashref instead of the element itself (see L</SYNOPSIS> for example)
 
 C<tidy> : the function will return tidy XML
@@ -817,8 +753,6 @@ C<xslt> : will add a <?xml-stylesheet?> link in the XML that's being output, of
 
 C<arrayref> : the function will create a simple arrayref instead of a simple hashref (which will preserve order and elements with duplicate tags)
 
-C<utf8> : the strings which will be returned will have their utf8 flag set (defaults to 0 for compatibility with software built with older versions of this module). The way this module works is that it holds everything in byte format internally (even if you provide it with a utf8 XML string), and then produces utf8 strings or simple structures if (and only if) asked for with this flag. UTF is an important issue, please read C<perldoc utf8> for more.
-
 =head1 FUNCTIONS
 
 =head2 xml_escape($string)
@@ -829,19 +763,19 @@ Returns the same string, but with the C<< < >>, C<< > >>, C<< & >>, C<< " >> and
 
 Returns the XML string in a tidy format (with tabs & newlines)
 
-Optional flags: C<file>, C<complete>, C<indentstring>, C<soft>, C<save>, C<utf8>
+Optional flags: C<file>, C<complete>, C<indentstring>, C<save>
 
 =head2 xml_to_object($raw_xml)
 
 Creates an 'XML::MyXML::Object' object from the raw XML provided
 
-Optional flags: C<file>, C<soft>
+Optional flags: C<file>
 
 =head2 object_to_xml($object)
 
 Creates an XML string from the 'XML::MyXML::Object' object provided
 
-Optional flags: C<complete>, C<tidy>, C<indentstring>, C<save>, C<utf8>
+Optional flags: C<complete>, C<tidy>, C<indentstring>, C<save>
 
 =head2 simple_to_xml($simple_array_ref)
 
@@ -851,7 +785,9 @@ Produces a raw XML string from either an array reference, a hash reference or a
     [ thing => [ name => 'John', location => [ city => 'New York', country => 'U.S.A.' ] ] ]
     { thing => { name => 'John', location => [ city => 'New York', city => 'Boston', country => 'U.S.A.' ] } }
 
-Optional flags: C<complete>, C<tidy>, C<indentstring>, C<save>, C<xslt>, C<utf8>
+All the strings in C<$simple_array_ref> need to contain characters, rather than bytes/octets. The XML output of this function however will be a UTF-8 encoded string (i.e. will contain bytes/octets).
+
+Optional flags: C<complete>, C<tidy>, C<indentstring>, C<save>, C<xslt>
 
 =head2 xml_to_simple($raw_xml)
 
@@ -859,11 +795,13 @@ Produces a very simple hash object from the raw XML string provided. An example
 
 Since the object created is a hashref, duplicate keys will be discarded. WARNING: This function only works on very simple XML strings, i.e. children of an element may not consist of both text and elements (child elements will be discarded in that case)
 
-Optional flags: C<internal>, C<strip>, C<file>, C<soft>, C<strip_ns>, C<arrayref>, C<utf8>
+All strings contained in the output simple structure, will contain characters rather than octets/bytes.
+
+Optional flags: C<internal>, C<strip>, C<file>, C<strip_ns>, C<arrayref>
 
 =head2 check_xml($raw_xml)
 
-Returns 1 if the $raw_xml string is valid XML (valid enough to be used by this module), and 0 otherwise.
+Returns true if the $raw_xml string is valid XML (valid enough to be used by this module), and false otherwise.
 
 Optional flags: C<file>
 
@@ -877,52 +815,45 @@ Returns the element specified by the path as an XML::MyXML::Object object. When
 
 When the element represented by the $obj object has only text contents, returns those contents as a string. If the $obj element has no contents, value will return an empty string.
 
-Optional flags: C<strip>, C<utf8>
+Optional flags: C<strip>
 
 =head2 $obj->attr('attrname' [, 'attrvalue'])
 
 Gets/Sets the value of the 'attrname' attribute of the top element. Returns undef if attribute does not exist. If called without the 'attrname' paramter, returns a hash with all attribute => value pairs. If setting with an attrvalue of C<undef>, then removes that attribute entirely.
 
-Optional flags: C<utf8>
+Input parameters and output are all in character strings, rather than octets/bytes.
+
+Optional flags: none
 
 =head2 $obj->tag
 
-Returns the tag of the $obj element (after stripping it from namespaces, unless the C<strip_ns> option is passed as false). E.g. if $obj represents an <rss:item> element, C<< $obj->tag >> will just return the name 'item'.
+Returns the tag of the $obj element. E.g. if $obj represents an <rss:item> element, C<< $obj->tag >> will return the string 'rss:item'.
 Returns undef if $obj doesn't represent a tag.
 
-Optional flags: C<utf8>, C<strip_ns>
+Optional flags: C<strip_ns>
 
 =head2 $obj->simplify
 
 Returns a very simple hashref, like the one returned with C<&XML::MyXML::xml_to_simple>. Same restrictions and warnings apply.
 
-Optional flags: C<internal>, C<strip>, C<strip_ns>, C<arrayref>, C<utf8>
+Optional flags: C<internal>, C<strip>, C<strip_ns>, C<arrayref>
 
 =head2 $obj->to_xml
 
 Returns the XML string of the object, just like calling C<object_to_xml( $obj )>
 
-Optional flags: C<complete>, C<tidy>, C<indentstring>, C<save>, C<utf8>
+Optional flags: C<complete>, C<tidy>, C<indentstring>, C<save>
 
 =head2 $obj->to_tidy_xml
 
 Returns the XML string of the object in tidy form, just like calling C<tidy_xml( object_to_xml( $obj ) )>
 
-Optional flags: C<complete>, C<indentstring>, C<save>, C<utf8>
-
-=head2 $obj->delete
-
-Deletes the object and all its children from memory. This is the only way to remove an XML object from memory and clear the RAM, since children and parents refer to each other circularly.
-
-The way it works is by removing references from the object's descendants to their parents.
+Optional flags: C<complete>, C<indentstring>, C<save>
 
 =head1 BUGS
 
-If you don't have a Github account to report your issues at
-L<https://github.com/akarelas/xml-myxml/issues>,
-then feel free to report any bugs or feature requests to
-C<bug-xml-myxml at rt.cpan.org>, or through the web interface at
-L<http://rt.cpan.org/NoAuth/ReportBug.html?Queue=XML-MyXML>.
+If you have a Github account, report your issues at
+L<https://github.com/akarelas/xml-myxml/issues>.
 I will be notified, and then you'll automatically be notified of progress on
 your bug as I make changes.
 
